// +build go1.13

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.
// Code generated by Microsoft (R) AutoRest Code Generator.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package armagfood

import (
	"context"
	"errors"
	"fmt"
	"github.com/Azure/azure-sdk-for-go/sdk/armcore"
	"github.com/Azure/azure-sdk-for-go/sdk/azcore"
	"net/http"
	"net/url"
	"strconv"
	"strings"
)

// FarmBeatsExtensionsClient contains the methods for the FarmBeatsExtensions group.
// Don't use this type directly, use NewFarmBeatsExtensionsClient() instead.
type FarmBeatsExtensionsClient struct {
	con *armcore.Connection
}

// NewFarmBeatsExtensionsClient creates a new instance of FarmBeatsExtensionsClient with the specified values.
func NewFarmBeatsExtensionsClient(con *armcore.Connection) *FarmBeatsExtensionsClient {
	return &FarmBeatsExtensionsClient{con: con}
}

// Get - Get farmBeats extension.
// If the operation fails it returns the *ErrorResponse error type.
func (client *FarmBeatsExtensionsClient) Get(ctx context.Context, farmBeatsExtensionID string, options *FarmBeatsExtensionsGetOptions) (FarmBeatsExtensionResponse, error) {
	req, err := client.getCreateRequest(ctx, farmBeatsExtensionID, options)
	if err != nil {
		return FarmBeatsExtensionResponse{}, err
	}
	resp, err := client.con.Pipeline().Do(req)
	if err != nil {
		return FarmBeatsExtensionResponse{}, err
	}
	if !resp.HasStatusCode(http.StatusOK) {
		return FarmBeatsExtensionResponse{}, client.getHandleError(resp)
	}
	return client.getHandleResponse(resp)
}

// getCreateRequest creates the Get request.
func (client *FarmBeatsExtensionsClient) getCreateRequest(ctx context.Context, farmBeatsExtensionID string, options *FarmBeatsExtensionsGetOptions) (*azcore.Request, error) {
	urlPath := "/providers/Microsoft.AgFoodPlatform/farmBeatsExtensionDefinitions/{farmBeatsExtensionId}"
	if farmBeatsExtensionID == "" {
		return nil, errors.New("parameter farmBeatsExtensionID cannot be empty")
	}
	urlPath = strings.ReplaceAll(urlPath, "{farmBeatsExtensionId}", url.PathEscape(farmBeatsExtensionID))
	req, err := azcore.NewRequest(ctx, http.MethodGet, azcore.JoinPaths(client.con.Endpoint(), urlPath))
	if err != nil {
		return nil, err
	}
	req.Telemetry(telemetryInfo)
	reqQP := req.URL.Query()
	reqQP.Set("api-version", "2020-05-12-preview")
	req.URL.RawQuery = reqQP.Encode()
	req.Header.Set("Accept", "application/json")
	return req, nil
}

// getHandleResponse handles the Get response.
func (client *FarmBeatsExtensionsClient) getHandleResponse(resp *azcore.Response) (FarmBeatsExtensionResponse, error) {
	var val *FarmBeatsExtension
	if err := resp.UnmarshalAsJSON(&val); err != nil {
		return FarmBeatsExtensionResponse{}, err
	}
	return FarmBeatsExtensionResponse{RawResponse: resp.Response, FarmBeatsExtension: val}, nil
}

// getHandleError handles the Get error response.
func (client *FarmBeatsExtensionsClient) getHandleError(resp *azcore.Response) error {
	body, err := resp.Payload()
	if err != nil {
		return azcore.NewResponseError(err, resp.Response)
	}
	errType := ErrorResponse{raw: string(body)}
	if err := resp.UnmarshalAsJSON(&errType); err != nil {
		return azcore.NewResponseError(fmt.Errorf("%s\n%s", string(body), err), resp.Response)
	}
	return azcore.NewResponseError(&errType, resp.Response)
}

// List - Get list of farmBeats extension.
// If the operation fails it returns the *ErrorResponse error type.
func (client *FarmBeatsExtensionsClient) List(options *FarmBeatsExtensionsListOptions) FarmBeatsExtensionListResponsePager {
	return &farmBeatsExtensionListResponsePager{
		pipeline: client.con.Pipeline(),
		requester: func(ctx context.Context) (*azcore.Request, error) {
			return client.listCreateRequest(ctx, options)
		},
		responder: client.listHandleResponse,
		errorer:   client.listHandleError,
		advancer: func(ctx context.Context, resp FarmBeatsExtensionListResponseResponse) (*azcore.Request, error) {
			return azcore.NewRequest(ctx, http.MethodGet, *resp.FarmBeatsExtensionListResponse.NextLink)
		},
		statusCodes: []int{http.StatusOK},
	}
}

// listCreateRequest creates the List request.
func (client *FarmBeatsExtensionsClient) listCreateRequest(ctx context.Context, options *FarmBeatsExtensionsListOptions) (*azcore.Request, error) {
	urlPath := "/providers/Microsoft.AgFoodPlatform/farmBeatsExtensionDefinitions"
	req, err := azcore.NewRequest(ctx, http.MethodGet, azcore.JoinPaths(client.con.Endpoint(), urlPath))
	if err != nil {
		return nil, err
	}
	req.Telemetry(telemetryInfo)
	reqQP := req.URL.Query()
	if options != nil && options.FarmBeatsExtensionIDs != nil {
		for _, qv := range options.FarmBeatsExtensionIDs {
			reqQP.Add("farmBeatsExtensionIds", qv)
		}
	}
	if options != nil && options.FarmBeatsExtensionNames != nil {
		for _, qv := range options.FarmBeatsExtensionNames {
			reqQP.Add("farmBeatsExtensionNames", qv)
		}
	}
	if options != nil && options.ExtensionCategories != nil {
		for _, qv := range options.ExtensionCategories {
			reqQP.Add("extensionCategories", qv)
		}
	}
	if options != nil && options.PublisherIDs != nil {
		for _, qv := range options.PublisherIDs {
			reqQP.Add("publisherIds", qv)
		}
	}
	if options != nil && options.MaxPageSize != nil {
		reqQP.Set("$maxPageSize", strconv.FormatInt(int64(*options.MaxPageSize), 10))
	}
	reqQP.Set("api-version", "2020-05-12-preview")
	req.URL.RawQuery = reqQP.Encode()
	req.Header.Set("Accept", "application/json")
	return req, nil
}

// listHandleResponse handles the List response.
func (client *FarmBeatsExtensionsClient) listHandleResponse(resp *azcore.Response) (FarmBeatsExtensionListResponseResponse, error) {
	var val *FarmBeatsExtensionListResponse
	if err := resp.UnmarshalAsJSON(&val); err != nil {
		return FarmBeatsExtensionListResponseResponse{}, err
	}
	return FarmBeatsExtensionListResponseResponse{RawResponse: resp.Response, FarmBeatsExtensionListResponse: val}, nil
}

// listHandleError handles the List error response.
func (client *FarmBeatsExtensionsClient) listHandleError(resp *azcore.Response) error {
	body, err := resp.Payload()
	if err != nil {
		return azcore.NewResponseError(err, resp.Response)
	}
	errType := ErrorResponse{raw: string(body)}
	if err := resp.UnmarshalAsJSON(&errType); err != nil {
		return azcore.NewResponseError(fmt.Errorf("%s\n%s", string(body), err), resp.Response)
	}
	return azcore.NewResponseError(&errType, resp.Response)
}
